package flm.b4a.accelerview;

import android.graphics.Bitmap;
import android.graphics.Bitmap.Config;
import android.graphics.Canvas;
import android.os.Build;
import android.view.MotionEvent;
import android.view.View;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.objects.ViewWrapper;
import anywheresoftware.b4a.objects.drawable.CanvasWrapper.BitmapWrapper;

@BA.ActivityObject
@BA.Author("F. Leneuf-Magaud")
@BA.Events(values = {
		"Draw(AC As AS_Canvas)", 
		"Touch(Action As Int, X As Int, Y As Int, Event As Object)",
		"Update(ElapsedTime As Long)"
})
@BA.ShortName("AcceleratedSurface")
@BA.Version(1.12F)
@SuppressWarnings("deprecation")
public class AcceleratedSurface extends ViewWrapper<AcceleratedView>
{
	/**
	 * Initializes the accelerated surface.
	 * @param IsHardwareAccelerated This setting should be set to true to benefit from the hardware acceleration.
	 * It is ignored by Android versions prior to Honeycomb (3.0, API 11).
	 */
	public void Initialize(final BA ba, final String EventPrefix, boolean IsHardwareAccelerated)
	{
		final String eventPrefix = EventPrefix.toLowerCase(BA.cul);
		innerInitialize(ba, eventPrefix, false);
		getObject().ba = ba;
		getObject().eventDraw = eventPrefix + "_draw";
		getObject().eventTouch = eventPrefix + "_touch";
		getObject().eventUpdate = eventPrefix + "_update";
		if (Integer.valueOf(Build.VERSION.SDK) >= Build.VERSION_CODES.HONEYCOMB) 
			setHwAc(IsHardwareAccelerated);
	}

	private void setHwAc(boolean IsAccelerated)
	{
		getObject().setLayerType(IsAccelerated ? View.LAYER_TYPE_HARDWARE:View.LAYER_TYPE_SOFTWARE, null);
	}

	@BA.Hide
	public void innerInitialize(final BA ba, final String eventPrefix, boolean keepOldObject)
	{
		if (!keepOldObject)
			setObject(new AcceleratedView(ba.context));
		super.innerInitialize(ba, eventPrefix, true);
		if (ba.subExists(eventPrefix + "_touch")) {
			((AcceleratedView)getObject()).setOnTouchListener(new View.OnTouchListener() {
				public boolean onTouch(View v, MotionEvent event)
				{
					ba.raiseEvent(getObject(), getObject().eventTouch,
							new Object[] { event.getAction(), (int)event.getX(), (int)event.getY(), event });
					return true;
				}
			});
		}
	}

	/**
	 * Gets whether the view is hardware accelerated.
	 * If the view is not hardware accelerated under Android 3 or +, check that you set IsHardwareAccelerated
	 * to true in Initialize and that you added SetApplicationAttribute(android:hardwareAccelerated, "true")
	 * to the manifest. 
	 */
	public boolean getIsHardwareAccelerated()
	{
		if (Integer.valueOf(Build.VERSION.SDK) >= Build.VERSION_CODES.HONEYCOMB) {
			return getObject().isHardwareAccelerated()
					&& getObject().getLayerType() != View.LAYER_TYPE_SOFTWARE;
		}
		else
			return false;
	}

	private Runnable TicTac;

	/**
	 * Fires automatically the Draw event at regular intervals (in milliseconds).
	 * This function will attempt to honor the requested interval, but the actual delay
	 * between Draw events may be different, depending on system load and capabilities.
	 */
	public void StartRegularDraw(BA ba, int Interval)
	{
		getObject().Last = System.currentTimeMillis();
		getObject().Delta = 0;
		TicTac = new TickTack(ba, getObject(), Interval, false);
		BA.handler.postDelayed(TicTac, Interval);
	}

	/**
	 * Fires automatically an Update event and a Draw event at regular intervals (in milliseconds).
	 * This function will attempt to honor the requested interval, but the actual delay
	 * between events may be different, depending on system load and capabilities.
	 */
	public void StartRegularUpdateAndDraw(BA ba, int Interval)
	{
		getObject().Last = System.currentTimeMillis();
		getObject().Delta = 0;
		TicTac = new TickTack(ba, getObject(), Interval, true);
		BA.handler.postDelayed(TicTac, Interval);
	}

	/**
	 * Stops the timer started with StartRegularDraw or StartRegularUpdateAndDraw.
	 */
	public void StopRegularDraw()
	{
		BA.handler.removeCallbacks(TicTac);
		getObject().Last = 0;
		getObject().Delta = 0;
	}

	/**
	 * Gets the elapsed time (in milliseconds) since the last Draw event.
	 * Returns always 0 if StartRegularDraw was not called.
	 */
	public long ElapsedTimeSinceLastDraw()
	{
		return getObject().Delta;
	}

	private static class TickTack implements Runnable
	{
		private final BA ba;
		private final AcceleratedView TimerAV;
		private final int TimerInterval;
		private final boolean CallUpdate;
		private final String UpdateEvent;
		private long NowUpdate, LastUpdate, DeltaUpdate;

		public TickTack(BA ba, AcceleratedView AV, int Interval, boolean Update)
		{
			this.ba = ba;
			TimerAV = AV;
			TimerInterval = Interval;
			CallUpdate = Update;
			if (Update && ba.subExists(AV.eventUpdate)) {
				UpdateEvent = AV.eventUpdate;
				LastUpdate = System.currentTimeMillis();
			}
			else
				UpdateEvent = "";
		}

		public void run()
		{
			BA.handler.postDelayed(this, TimerInterval);
			if (!ba.isActivityPaused())	{
				if (CallUpdate) {
					NowUpdate = System.currentTimeMillis();
					DeltaUpdate = NowUpdate - LastUpdate;
					LastUpdate = NowUpdate;
					ba.raiseEvent(TimerAV, UpdateEvent, new Object[] { DeltaUpdate });
				}
				TimerAV.invalidate();
			}
		}
	}

	/**
	 * Returns a screenshot of the current contents of the view.
	 */
	public BitmapWrapper getBitmap()
	{
		BitmapWrapper bmp = new BitmapWrapper();
		bmp.setObject(Bitmap.createBitmap(this.getWidth(), this.getHeight(), Config.ARGB_8888));
		Canvas cvs = new Canvas(bmp.getObject());
		this.getObject().draw(cvs);
		return bmp;
	}
}
