package flm.b4a.accelerview;

import android.graphics.Path;
import android.graphics.RectF;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.objects.drawable.CanvasWrapper.RectWrapper;

@BA.ShortName("AS_Path")
public class ComplexPath
{
	private boolean isInitialized = false;
	protected Path path = null;

	/**
	 * Initializes the AS_Path object and sets its beginning point to (x,y).
	 */
	public void Initialize(float x, float y)
	{
		path = new Path();
		path.moveTo(x, y);
		isInitialized = true;
	}

	private void CheckInitialization(String funcName) throws Exception
	{
		if (!isInitialized)
			throw(new Exception(funcName + ": path not initialized"));
	}

	/**
	 * Adds a copy of the given path to the current path.
	 */
	public final ComplexPath AddPath(ComplexPath PathToAppend) throws Exception
	{
		CheckInitialization("AddPath");
		path.addPath(PathToAppend.path);
		return this;
	}

	/**
	 * Appends the specified arc to the path as a new contour.
	 * An automatic LineTo is added to connect the current contour to the start of the arc.
	 * @param OvalRect The bounds of oval used to define the shape and size of the arc.
	 * @param StartAngle Starting angle (in degrees) where the arc begins.
	 * @param SweepAngle Sweep angle (in degrees) measured clockwise.
	 */
	public final ComplexPath ArcTo(RectWrapper OvalRect, float StartAngle, float SweepAngle) throws Exception
	{
		CheckInitialization("ArcTo");
		RectF R = new RectF(OvalRect.getObject());
		path.arcTo(R, StartAngle, SweepAngle);
		return this;
	}

	/**
	 * Closes the current contour. If the current point is not equal to the first point of the contour, a line segment is automatically added.
	 */
	public void Close() throws Exception
	{
		CheckInitialization("Close");
		path.close();
	}

	/**
	 * Computes the bounds of the control points of the path, and writes the answer into rctBounds.
	 */
	public void ComputeBounds(RectWrapper rctBounds) throws Exception
	{
		CheckInitialization("ComputeBounds");
		RectF rctB = new RectF();
		path.computeBounds(rctB, true);
		rctBounds.Initialize((int)rctB.left, (int)rctB.top, (int)rctB.right, (int)rctB.bottom);
	}

	/**
	 * Adds a cubic bezier from the last point, approaching control points (x1,y1) and (x2,y2), and ending at (x3,y3).
	 */
	public final ComplexPath CubicTo(float x1, float y1, float x2, float y2, float x3, float y3) throws Exception
	{
		CheckInitialization("CubicTo");
		path.cubicTo(x1, y1, x2, y2, x3, y3);
		return this;
	}

	/**
	 * Same as CubicTo, but the coordinates are considered relative to the current point on this contour.
	 */
	public final ComplexPath rCubicTo(float dx1, float dy1, float dx2, float dy2, float dx3, float dy3) throws Exception
	{
		CheckInitialization("rCubicTo");
		path.rCubicTo(dx1, dy1, dx2, dy2, dx3, dy3);
		return this;
	}

	/**
	 * Adds a line from the last point to the specified point (x,y).
	 */
	public final ComplexPath LineTo(float x, float y) throws Exception
	{
		CheckInitialization("LineTo");
		path.lineTo(x, y);
		return this;
	}

	/**
	 * Same as LineTo, but the coordinates are considered relative to the last point on this contour.
	 */
	public final ComplexPath rLineTo(float dx, float dy) throws Exception
	{
		CheckInitialization("rLineTo");
		path.rLineTo(dx, dy);
		return this;
	}

	/**
	 * Sets the beginning of the next contour to the point (x,y).
	 */
	public final ComplexPath MoveTo(float x, float y) throws Exception
	{
		CheckInitialization("MoveTo");
		path.moveTo(x, y);
		return this;
	}

	/**
	 * Sets the beginning of the next contour relative to the last point on the previous contour.
	 */
	public final ComplexPath rMoveTo(float dx, float dy) throws Exception
	{
		CheckInitialization("rMoveTo");
		path.rMoveTo(dx, dy);
		return this;
	}

	/**
	 * Adds a quadratic bezier from the last point, approaching control point (x1,y1), and ending at (x2,y2).
	 */
	public final ComplexPath QuadTo(float x1, float y1, float x2, float y2) throws Exception
	{
		CheckInitialization("QuadTo");
		path.quadTo(x1, y1, x2, y2);
		return this;
	}

	/**
	 * Same as QuadTo, but the coordinates are considered relative to the last point on this contour.
	 */
	public final ComplexPath rQuadTo(float dx1, float dy1, float dx2, float dy2) throws Exception
	{
		CheckInitialization("rQuadTo");
		path.rQuadTo(dx1, dy1, dx2, dy2);
		return this;
	}

	/**
	 * Clears any lines and curves from the path, making it empty.
	 */
	public void Reset() throws Exception
	{
		CheckInitialization("Reset");
		path.reset();
	}

	public final static Path.FillType FILLTYPE_EVENODD = Path.FillType.EVEN_ODD;
	public final static Path.FillType FILLTYPE_INVERSE_EVENODD = Path.FillType.INVERSE_EVEN_ODD;
	public final static Path.FillType FILLTYPE_INVERSE_WINDING = Path.FillType.INVERSE_WINDING;
	public final static Path.FillType FILLTYPE_WINDING = Path.FillType.WINDING;

	/**
	 * Sets the path's fill type.
	 * @param FT One of the FILLTYPE constants.
	 */
	public ComplexPath SetFillType(Path.FillType FT) throws Exception
	{
		CheckInitialization("SetFillType");
		path.setFillType(FT);
		return this;
	}

	/**
	 * Toggles the INVERSE state of the fill type.
	 */
	public void ToggleInverseFillType() throws Exception
	{
		CheckInitialization("ToggleInverseFillType");
		path.toggleInverseFillType();
	}

	/**
	 * Imports a Path from another library.
	 */
	public void ImportPath(Path p) throws Exception
	{
		CheckInitialization("ImportPath");
		path = p;
	}
	@BA.Hide
	public Path getPath()
	{
		if (isInitialized)
			return path;
		else
			return null;
	}

	public boolean getIsInitialized()
	{
		return isInitialized;
	}
}
