package flm.b4a.accelerview;

import android.graphics.Paint;
import android.graphics.Rect;
import android.graphics.Paint.Style;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.keywords.constants.TypefaceWrapper;

@BA.ShortName("AS_Text")
public class ComplexText
{
	private boolean isInitialized = false;
	private ComplexObject objText = new ComplexObject();

	/**
	 * Creates a text object.
	 * @param Typeface Typeface (font) to use.
	 * @param Filter Whether the edges are smoothed out.
	 */
	public final ComplexText CreateText(TypefaceWrapper Typeface, boolean Filter)
	{
		if (objText.CreateText(Filter) == null)
			return null;
		objText.paint.setTypeface(Typeface.getObject());
		isInitialized = true;
		return this;
	}

	private void CheckInitialization(String funcName) throws Exception
	{
		if (!isInitialized)
			throw(new Exception(funcName + ": object not initialized"));
	}

	/**
	 * Clones an existing AS_Text.
	 */
	public final ComplexText Clone(ComplexText ObjToCopy)
	{
		isInitialized = false;
		if (!ObjToCopy.isInitialized)
			return null;
		objText.Clone(ObjToCopy.getCO());
		isInitialized = objText.getIsInitialized();
		return this;
	}

	/**
	 * Replaces the typeface (font).
	 */
	public void ReplaceTypeface(TypefaceWrapper NewTypeface) throws Exception
	{
		CheckInitialization("ReplaceTypeface");
		objText.paint.setTypeface(NewTypeface.getObject());
	}

	/**
	 * Sets the alpha value of the text color, leaving its R,G,B values unchanged.
	 */
	public void SetAlpha(int Alpha) throws Exception
	{
		CheckInitialization("SetAlpha");
		objText.paint.setAlpha(Alpha);
	}
	/**
	 * Returns the alpha value of the text color.
	 */
	public int GetAlpha() throws Exception
	{
		CheckInitialization("GetAlpha");
		return objText.paint.getAlpha();
	}

	/**
	 * Paints the text with the given color.
	 * If this function is called after SetTexture, SetTexture is ignored. 
	 */
	public final ComplexText SetColor(int Color) throws Exception
	{
		CheckInitialization("SetColor");
		objText.SetColor(Color);
		return this;
	}
	/**
	 * Gets the text color.
	 */
	public int GetColor() throws Exception
	{
		CheckInitialization("GetColor");
		return objText.paint.getColor();
	}

	/**
	 * Sets the text size. This value is automatically scaled so do not scale it yourself.
	 */
	public final ComplexText SetSize(float TextSize) throws Exception
	{
		CheckInitialization("SetSize");
		objText.paint.setTextSize(TextSize * BA.applicationContext.getResources().getDisplayMetrics().scaledDensity);
		return this;
	}
	/**
	 * Gets the text size.
	 */
	public float GetSize() throws Exception
	{
		CheckInitialization("GetSize");
		return objText.paint.getTextSize();
	}

	public static final Style STYLE_FILL = Style.FILL;
	public static final Style STYLE_FILL_AND_STROKE = Style.FILL_AND_STROKE;
	public static final Style STYLE_STROKE = Style.STROKE;

	/**
	 * Defines whether the text will be filled or stroked, or both.
	 * @param Style One of the STYLE constants.
	 * @param StrokeWidth The stroke width. Relevant when Style &lt;&gt; STYLE_FILL.
	 */
	public final ComplexText SetStyle(Style style, int StrokeWidth) throws Exception
	{
		CheckInitialization("SetStyle");
		objText.SetStyle(style, StrokeWidth);
		return this;
	}

	/**
	 * Adds a colorfilter that multiplies the RGB channels by one color, and then adds a second color,
	 * pinning the result for each component to [0..255].
	 */
	public final ComplexText SetLightingColorFilter(int Multiply, int Add) throws Exception
	{
		CheckInitialization("SetLightingColorFilter");
		objText.SetLightingColorFilter(Multiply, Add);
		return this;
	}

	/**
	 * Sets the path along which the text will be drawn.
	 * Warning: Nothing will be drawn under Honeycomb or ICS if the hardware acceleration is enabled.
	 * @param Path An instance of the Path class or an instance of the AS_Path class.
	 */
	public final ComplexText SetPath(Object Path) throws Exception
	{
		CheckInitialization("SetPath");
		if (Path == null)
			objText.pathParam = null;
		else if (Path.getClass().getName().equals("android.graphics.Path"))
			objText.pathParam = (android.graphics.Path) Path;
		else if (Path instanceof ComplexPath) 
			objText.pathParam = ((ComplexPath)Path).path;
		else
			return null;
		return this;
	}

	/**
	 * Sets the text's horizontal scale factor.
	 */
	public final ComplexText SetScaleX(float ScaleX) throws Exception
	{
		CheckInitialization("SetScaleX");
		objText.paint.setTextScaleX(ScaleX);
		return this;
	}

	/**
	 * Sets the text's horizontal skew factor.
	 */
	public final ComplexText SetSkewX(float SkewX) throws Exception
	{
		CheckInitialization("SetSkewX");
		objText.paint.setTextSkewX(SkewX);
		return this;
	}

	/**
	 * Adds a shadow layer below the main layer, with the specified offset, blur radius and color.
	 * If radius is 0, then the shadow layer is removed.
	 */
	public final ComplexText SetShadow(float dX, float dY, float Radius, int Color) throws Exception
	{
		CheckInitialization("SetShadow");
		objText.paint.setShadowLayer(Radius, dX, dY, Color);
		return this;
	}

	/**
	 * Paints the text with the given texture.
	 * If this function is called after SetColor, SetColor is ignored. 
	 */
	public final ComplexText SetTexture(Texture texture) throws Exception
	{
		CheckInitialization("SetTexture");
		objText.SetTexture(texture);
		return this;
	}

	/**
	 * Enables or disables bold text.
	 */
	public final ComplexText SetBold(boolean Enabled) throws Exception
	{
		CheckInitialization("SetBold");
		objText.paint.setFakeBoldText(Enabled);
		return this;
	}

	/**
	 * Enables or disables underline text.
	 * This may not work as expected depending on the OS version and the chosen style. 
	 */
	public final ComplexText SetUnderline(boolean Enabled) throws Exception
	{
		CheckInitialization("SetUnderline");
		objText.paint.setUnderlineText(Enabled);
		return this;
	}

	/**
	 * Returns the width of the given text.
	 */
	public float MeasureStringWidth(String Text)
	{
		return objText.paint.measureText(Text);
	}

	/**
	 * Returns the height of the given text.
	 */
	public int MeasureStringHeight(String Text)
	{
		Rect r = new Rect();
		objText.paint.getTextBounds(Text, 0, Text.length(), r);
		return r.height();
	}

	protected ComplexObject getCO()
	{
		return objText;
	}

	/**
	 * Gets/sets the internal Paint.
	 */
	public Paint getPaint() throws Exception
	{
		CheckInitialization("getPaint");
		return objText.paint;
	}
	public void setPaint(Paint paint) throws Exception
	{
		CheckInitialization("setPaint");
		objText.paint = paint;
	}

	public boolean getIsInitialized()
	{
		return isInitialized;
	}
}
