package flm.b4a.accelerview;

import java.io.IOException;
import java.io.InputStream;

import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.ColorMatrix;
import android.graphics.ColorMatrixColorFilter;
import android.graphics.Paint;
import android.graphics.drawable.Drawable;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.objects.collections.Map;
import anywheresoftware.b4a.objects.drawable.CanvasWrapper.BitmapWrapper;
import anywheresoftware.b4a.objects.streams.File;

@BA.ShortName("AS_ImageUtils")
public class ImageUtils
{
	/**
	 * Returns a Map containing the dimensions of the given image (keys = "Height" and "Width").
	 * The image is not loaded in memory.
	 * This function does not work with images in the assets folder. 
	 */
	public static Map GetImageDimensions(String Dir, String FileName)
	{
		Map m = new Map();
		m.Initialize();
		BitmapFactory.Options options = new BitmapFactory.Options();
		options.inJustDecodeBounds = true;
		BitmapFactory.decodeFile(File.Combine(Dir, FileName), options);
		if (options.outHeight > 0 || options.outWidth > 0) {
			m.Put("Height", options.outHeight);
			m.Put("Width", options.outWidth);
		}
		return m;
	}

	/**
	 * Loads and rescales a bitmap.
	 * @param dstWidth The new bitmap's desired width or -1 to keep the original width.
	 * @param dstHeight The new bitmap's desired height or -1 to keep the original height.
	 * @param Filter If True, improves the sampling quality.
	 */
	public static Bitmap LoadScaledBitmap(String Dir, String FileName, int dstWidth, int dstHeight, boolean Filter) throws IOException
	{
		File.InputStreamWrapper in = File.OpenInput(Dir, FileName);
		BitmapFactory.Options opt = new BitmapFactory.Options();
		opt.inJustDecodeBounds = true;
		BitmapFactory.decodeStream((InputStream)in.getObject(), null, opt);
		in.Close();

		int ratio = opt.outWidth / dstWidth;
		ratio = Math.max(ratio, opt.outHeight / dstHeight);
		BitmapFactory.Options opt2 = null;
		if (ratio > 1) {
			opt2 = new BitmapFactory.Options();
			opt2.inSampleSize = ratio;
		}
		in = File.OpenInput(Dir, FileName);
		Bitmap bmp;
		try {
			bmp = BitmapFactory.decodeStream((InputStream)in.getObject(), null, opt2);
		}
		catch (OutOfMemoryError OoMe) {
			throw new RuntimeException("LoadScaledBitmap: Out of memory.");
		}
		in.Close();
		if (bmp == null)
			throw new RuntimeException("LoadScaledBitmap: Error loading bitmap.");

		if (dstWidth == -1 && dstHeight == -1)
			return bmp;
		else {
			if (dstWidth == -1)
				dstWidth = bmp.getWidth();
			if (dstHeight == -1)
				dstHeight = bmp.getHeight();
			return Bitmap.createScaledBitmap(bmp, dstWidth, dstHeight, Filter);
		}
	}

	/**
	 * Creates a new bitmap scaled from an existing bitmap.
	 * If the specified width and height are the same as the current width and height of the source bitmap, the source bitmap is returned and no new bitmap is created.
	 * @param srcBmp The source bitmap.
	 * @param dstWidth The new bitmap's desired width.
	 * @param dstHeight The new bitmap's desired height.
	 * @param filter True if the source should be filtered.
	 */
	public static Bitmap CreateScaledBitmap(BitmapWrapper srcBmp, int dstWidth, int dstHeight, boolean filter)
	{
		return Bitmap.createScaledBitmap(srcBmp.getObject(), dstWidth, dstHeight, filter);
	}

	/**
	 * Creates a new bitmap by removing the outer parts of another one.
	 * @param srcBmp The source bitmap.
	 * @param x,y Coordinates of the cropping rectangle.
	 * @param width,height Size of the cropping rectangle.
	 */
	public static Bitmap Crop(BitmapWrapper srcBmp, int x, int y, int width, int height)
	{
		return Bitmap.createBitmap(srcBmp.getObject(), x, y, width, height); 
	}

	/**
	 * Converts a bitmap from the format ARGB_8888 to the format RGB_565 to save memory.
	 * The returned bitmap is mutable.
	 */
	public static Bitmap ReduceColors(BitmapWrapper srcBmp)
	{
		return srcBmp.getObject().copy(Bitmap.Config.RGB_565, true);
	}

	/**
	 * Returns a mutable bitmap from an immutable one.
	 */
	public static Bitmap MakeMutable(BitmapWrapper srcBmp)
	{
		return srcBmp.getObject().copy(srcBmp.getObject().getConfig(), true);
	}

	/**
	 * Returns in Pixels() a copy of the data in the bitmap. Each value is a packed int representing a Color.
	 * The stride parameter allows the caller to allow for gaps in the returned pixels array between rows.
	 * For normal packed results, just pass width for the stride value.
	 * @param Pixels The array to receive the bitmap's colors
	 * @param offset The first index to write into Pixels()
	 * @param stride The number of entries in Pixels() to skip between rows (must be >= bitmap's width). Can be negative.
	 * @param x The x coordinate of the first pixel to read from the bitmap
	 * @param y The y coordinate of the first pixel to read from the bitmap
	 * @param width The number of pixels to read from each row
	 * @param height The number of rows to read
	 */
	public static void GetPixels(BitmapWrapper src, int[] Pixels, int offset, int stride, int x, int y, int width, int height)
			throws IllegalArgumentException, ArrayIndexOutOfBoundsException
	{
		src.getObject().getPixels(Pixels, offset, stride, x, y, width, height);
	}

	/**
	 * Gets a drawable from the Android system resources.
	 */
	public static Drawable LoadSystemDrawable(BA ba, String Name)
	{
		int ID_drawable = Resources.getSystem().getIdentifier(Name, "drawable", "android");
		return ba.context.getResources().getDrawable(ID_drawable);
	}

	/**
	 * Gets a 9-patch drawable from the application resources.
	 */
	public static Drawable LoadNinePatchDrawable(BA ba, String Name)
	{
		int ID_drawable = ba.context.getResources().getIdentifier(Name, "drawable", ba.context.getPackageName());
		return ba.context.getResources().getDrawable(ID_drawable);
	}

	/**
	 * Changes the transparency, hue and saturation of the given bitmap, and returns the result.
	 * The bitmap must be mutable.
	 * @param Alpha From -1 to 255. -1 = no change, 0 = transparent, 255 = opaque.
	 * @param Hue From 0 to 360. 0 or 360 = no change.
	 * @param Saturation 0 = gray-scale, 1 = no change.
	 */
	public static Bitmap AlterColors(BitmapWrapper srcBmp, int Alpha, int Hue, float Saturation)
	{
		ColorMatrix cm = new ColorMatrix();
		if (Saturation != 1)
			cm.setSaturation(Saturation);
		if (Hue != 0) {
			final float NewHue = (float) Hue / 180f * (float) Math.PI;
			final float cosVal = (float) Math.cos(NewHue);
			final float sinVal = (float) Math.sin(NewHue);
			final float lumR = 0.213f;
			final float lumG = 0.715f;
			final float lumB = 0.072f;
			float[] mat = new float[] { 
					lumR + cosVal * (1 - lumR) + sinVal * (-lumR), lumG + cosVal * (-lumG) + sinVal * (-lumG), lumB + cosVal * (-lumB) + sinVal * (1 - lumB), 0, 0, 
					lumR + cosVal * (-lumR) + sinVal * (0.143f), lumG + cosVal * (1 - lumG) + sinVal * (0.140f), lumB + cosVal * (-lumB) + sinVal * (-0.283f), 0, 0,
					lumR + cosVal * (-lumR) + sinVal * (-(1 - lumR)), lumG + cosVal * (-lumG) + sinVal * (lumG), lumB + cosVal * (1 - lumB) + sinVal * (lumB), 0, 0, 
					0f, 0f, 0f, 1f, 0f, 
					0f, 0f, 0f, 0f, 1f };
			cm.postConcat(new ColorMatrix(mat));
		}
		Bitmap bmp = Bitmap.createBitmap(srcBmp.getObject());
		Canvas canvas = new Canvas(bmp);
		if (Saturation != 1 || Hue != 0) { 
			Paint p = new Paint();
			ColorMatrixColorFilter cmFilter = new ColorMatrixColorFilter(cm);
			p.setColorFilter(cmFilter);
			canvas.drawBitmap(bmp, 0.0F, 0.0F, p);
		}
		if (Alpha >= 0) {
			int Color = (Alpha & 0xFF) << 24;
			canvas.drawColor(Color, android.graphics.PorterDuff.Mode.DST_IN);
		}
		return bmp;
	}

	/**
	 * Gets the density of the given bitmap.
	 */
	public static int GetDensity(BitmapWrapper srcBmp)
	{
		return srcBmp.getObject().getDensity();
	}

	/**
	 * Changes the density of the given bitmap.
	 */
	public static void SetDensity(BitmapWrapper srcBmp, int Density)
	{
		srcBmp.getObject().setDensity(Density);
	}
}
