package flm.b4a.accelerview;

import android.graphics.BitmapShader;
import android.graphics.ComposeShader;
import android.graphics.LinearGradient;
import android.graphics.Matrix;
import android.graphics.PorterDuff.Mode;
import android.graphics.RadialGradient;
import android.graphics.Shader;
import android.graphics.SweepGradient;
import android.graphics.Shader.TileMode;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.objects.drawable.CanvasWrapper.BitmapWrapper;

@BA.ShortName("AS_Texture")
public class Texture
{
	public final static Shader.TileMode TILEMODE_CLAMP = TileMode.CLAMP; 
	public final static Shader.TileMode TILEMODE_REPEAT = TileMode.REPEAT; 
	public final static Shader.TileMode TILEMODE_MIRROR = TileMode.MIRROR; 

	public final static Mode MERGEMODE_DARKEN = Mode.DARKEN;
	public final static Mode MERGEMODE_LIGHTEN = Mode.LIGHTEN;
	public final static Mode MERGEMODE_MULTIPLY = Mode.MULTIPLY;
	public final static Mode MERGEMODE_SCREEN = Mode.SCREEN;
	public final static Mode MERGEMODE_TEX1_ATOP = Mode.SRC_ATOP;
	public final static Mode MERGEMODE_TEX1_IN = Mode.SRC_IN;
	public final static Mode MERGEMODE_TEX1_OUT = Mode.SRC_OUT;
	public final static Mode MERGEMODE_TEX1_OVER = Mode.SRC_OVER;
	public final static Mode MERGEMODE_TEX2_ATOP = Mode.DST_ATOP;
	public final static Mode MERGEMODE_TEX2_IN = Mode.DST_IN;
	public final static Mode MERGEMODE_TEX2_OUT = Mode.DST_OUT;
	public final static Mode MERGEMODE_TEX2_OVER = Mode.DST_OVER;
	public final static Mode MERGEMODE_XOR = Mode.XOR;

	private Shader shader = null;
	private Matrix m;
	private boolean isInitialized = false;

	public Texture()
	{
	}

	/**
	 * Creates a texture with the given bitmap. This texture can be repeated or mirrored.
	 * @param Image a bitmap containing the texture image.
	 * @param ModeX,ModeY one of the TILEMODE constants for each axis.
	 */
	public final Texture CreateWithBitmap(BitmapWrapper Image, Shader.TileMode ModeX, Shader.TileMode ModeY) throws Exception
	{
		if (Image == null || !Image.IsInitialized())
			throw(new Exception("CreateWithBitmap: Image not initialized"));
		shader = new BitmapShader(Image.getObject(), ModeX, ModeY);
		m = new Matrix();
		isInitialized = true;
		return this;
	}

	/**
	 * Creates a texture with a linear gradient. This texture can be repeated or mirrored.
	 * @param X0 The x-coordinate for the start of the gradient line.
	 * @param Y0 The y-coordinate for the start of the gradient line.
	 * @param X1 The x-coordinate for the end of the gradient line.
	 * @param Y1 The y-coordinate for the end of the gradient line.
	 * @param Colors The colors to be distributed along the gradient line. There must be at least 2 colors in the array.
	 * @param TileMode one of the TILEMODE constants.
	 */
	public final Texture CreateLinearGradient(float X0, float Y0, float X1, float Y1, int Colors[], Shader.TileMode TileMode)
	{
		shader = new LinearGradient(X0, Y0, X1, Y1, Colors, null, TileMode);
		m = new Matrix();
		isInitialized = true;
		return this;
	}

	/**
	 * Creates a texture with a radial gradient. This texture can be repeated or mirrored.
	 * @param X The x-coordinate of the center of the circle.
	 * @param Y The y-coordinate of the center of the circle.
	 * @param Radius Must be positive. The radius of the circle for this gradient.
	 * @param Colors The colors to be distributed between the center and edge of the circle. There must be at least 2 colors in the array.
	 * @param TileMode one of the TILEMODE constants.
	 */
	public final Texture CreateRadialGradient(float X, float Y, float Radius, int Colors[], Shader.TileMode TileMode)
	{
		shader = new RadialGradient(X, Y, Radius, Colors, null, TileMode);
		m = new Matrix();
		isInitialized = true;
		return this;
	}

	/**
	 * Creates a texture with a sweep gradient.
	 * @param X The x-coordinate of the center.
	 * @param Y The y-coordinate of the center.
	 * @param Colors The colors to be distributed around the center. There must be at least 2 colors in the array.
	 */
	public final Texture CreateSweepGradient(float X, float Y, int Colors[])
	{
		shader = new SweepGradient(X, Y, Colors, null);
		m = new Matrix();
		isInitialized = true;
		return this;
	}

	/**
	 * Translates the origin point.
	 */
	public void Translate(int OffsetX, int OffsetY) throws Exception
	{
		if (!isInitialized)
			throw(new Exception("Translate: texture not initialized"));
		if (shader != null) {
			m.setTranslate(OffsetX, OffsetY);
			shader.setLocalMatrix(m);
		}
	}

	/**
	 * Merges two textures of different types (a bitmap and a radial gradient for example).
	 * The returned texture cannot be merged with another texture.
	 * @param Tex1,Tex2 the textures to merge. They must be of different types.
	 * @param MergeMode one of the MERGEMODE constants.
	 */
	public final Texture Merge(Texture Tex1, Texture Tex2, Mode MergeMode)
	{
		if (Tex1 == null)
			return Tex2;
		if (Tex2 == null)
			return Tex1;
		shader = new ComposeShader(Tex1.getShader(), Tex2.getShader(), MergeMode);
		m = new Matrix();
		isInitialized = true;
		return this;
	}

	@BA.Hide
	public Shader getShader()
	{
		return shader;
	}

	public boolean getIsInitialized()
	{
		return isInitialized;
	}
}
